package l03_arrays;

//	Since J2SE 5.0 variable arguments' language feature makes it possible to call
//	a method with a variable number of arguments. Before making that call, the
//	rightmost parameter in the method's parameter list must conform to the
//	following syntax:

//	type ... variableName

//	The ellipsis (...) identifies a variable number of arguments.

//	Variable arguments is implemented in terms of arrays. (numbers.length and
//	numbers[i] are the giveaways.) It turns out that the ellipsis is just
//	syntactic sugar for having the compiler create and initialize an array of
//	same-typed values and pass that array's reference to a method. For example,
//	the compiler converts the method call sum(10, 20) into an equivalent
//	sum(new int [] {10, 20}) method call.

public class VariadicFunctions {

	public static void main (String [] args) {

		System.out.println("Average: " + avg (20.3, 3.1415, 32.3));
		System.out.println("Concatenation: " + concat ("Hello", " ", "World"));
		System.out.println("Maximum: " + max (30, 22.3, -9.3, 173.2));
		System.out.println("Minimum: " + min (30, 22.3, -9.3, 173.2));
		System.out.println("Sum: " + sum (20, 30));
	}

	private static double avg (double ... numbers) {
		double total = 0;
		for (double number : numbers) total += number;
		return total / numbers.length;
	}

	private static String concat (String ... strings) {
		StringBuilder sb = new StringBuilder ();
		for (String string : strings) {
			sb.append(string);
		}
		return sb.toString ();
	}

	private static double max (double ... numbers) {
		double maximum = Double.MIN_VALUE;
		for (double number : numbers) {
			maximum = Math.max(maximum, number);
		}
		return maximum;
	}

	private static double min (double ... numbers) {
		double minimum = Double.MAX_VALUE;
		for (double number : numbers) {
			minimum = Math.min(minimum, number);
		}
		return minimum;
	}

	private static int sum (int ... numbers) {
		int total = 0;
		for (int number : numbers) {
			total += number;
		}
		return total;
	}
}
